#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu May 12 13:11:10 2022

@author: stevenheuer
"""

import numpy as np

import matplotlib.pyplot as plt
from matplotlib import rcParams
import matplotlib.transforms as transforms
import matplotlib as mpl
from mpl_toolkits.axes_grid1 import make_axes_locatable, axes_size, ImageGrid
from matplotlib.ticker import AutoMinorLocator

import pandas as pd
from pandas import read_csv

from scipy.signal import savgol_filter
from scipy.ndimage import gaussian_filter
from sympy.functions.special.tensor_functions import eval_levicivita
from scipy.io import readsav

from scipy.interpolate import interp1d
from pytplot import *

import matplotlib.gridspec as gridspec

rcParams['text.usetex'] = True

mpl.rc('font', family='serif')
mpl.rcParams.update({
        "pgf.rcfonts"  : False,
        "pgf.preamble" : '\\usepackage{fontspec}\\usepackage[T1]{fontenc}\\usepackage[utf8x]{inputenc}\\usepackage[light]{kpfonts}',
})
#%%

#import pathlib
from pathlib import Path
fpath = Path(__file__).parent.resolve()

Ay = readsav(Path.joinpath(fpath,'Ay_t50.sav'))

data = readsav(Path.joinpath(fpath,'simulation_t50.sav'))

dat = data['data']
july11_vprobe = pd.read_csv(Path.joinpath(fpath,'tkmnakamura_virtual_mms1_data_20170711_for_roy.txt'),
                     delimiter='    ',usecols=[0,1,2],names=['x','y','z'],)

aug10_vprobe = pd.read_csv(Path.joinpath(fpath,'aug10_trajectory.csv'),
                     delimiter=',',usecols=[0,1],names=['x','z'],)

june17_vprobe = pd.read_csv(Path.joinpath(fpath,'june17_trajectory.csv'),
                     delimiter=',',usecols=[0,1],names=['x','z'],)


#%%

convert = True

if convert == True:
   nb=0.03
   bb=12.
   vaeb=bb*1e-9/np.sqrt(4*np.pi*1e-7*nb*1e6*9.11e-31)/1000.
   bconvert=bb
   jconvert=1.6022*nb*vaeb
   rconvert=5.31/np.sqrt(3*nb)
   econvert=bb*vaeb/1000.
   vconvert=vaeb
   nconvert=nb*3
else:
   bconvert=1.
   jconvert=1.
   rconvert=1.
   econvert=1.
   vconvert=1.
   nconvert=1.
   
nb=0.03
bb=12.
vaeb=bb*1e-9/np.sqrt(4*np.pi*1e-7*nb*1e6*9.11e-31)/1000.   

rconvert=5.31/np.sqrt(3*nb)
#vpx,vpz = july11_vprobe['x']/rconvert,july11_vprobe['z']/rconvert


#-160de(=-8di) to 160de(=8di) in x
#-40de(=-2di) to 40de(=2di) in z

xd = dat[0:,0]
zd = dat[0:,1]

xrange = [np.min(xd),np.max(xd)]
zrange = [np.min(zd),np.max(zd)]

xlim =[-190, 130]
zlim =[-40, 40]

xind = np.where(np.abs(xd) <= xrange[1])[0]
zind = np.where(np.abs(zd) <= zrange[1])[0]#% and np.where( np.abs(xd) <= xrange[1])[0]

xdiff_ind = np.where(np.diff(xd[xind])==(np.min(xd[xind])-np.max(xd[xind])))[0]
zdiff_ind = np.where(np.diff(zd[zind])!=0)[0]

zd_uniq = np.unique(zd)
xd_uniq = np.unique(xd)

zlim_ind = np.where(np.abs(zd_uniq)<= zlim[1])[0]
xlim_ind = np.where(np.abs(xd_uniq) <= xlim[1])[0]


def simData(datarr):
    arr = []
    for i in range(1,len(xdiff_ind)):    #which x indices within the window correspond to z indices within the window 
    
        xs = datarr[xdiff_ind[i-1]:xdiff_ind[i]]
        arr.append(xs)
    arr = np.asarray(arr)   
    
    if xlim == xrange:
        return arr
    else:
        arr2 = arr[zlim_ind]
        arr3 = []
        for ind in xlim_ind:
            arr3.append(arr2[0:,ind])
        arr3 = np.asarray(arr3).T    
            
        return np.asarray(arr3)
  
X, Y = np.meshgrid(Ay['xx_t50']-30.5,Ay['zz_t50'])

xx = simData(dat[0:,0])[0]-30.5
zz = simData(dat[0:,1])[0:,0]
bx_sim = simData(dat[0:,2])*bconvert
by_sim = simData(dat[0:,3])*bconvert
bz_sim = simData(dat[0:,4])*bconvert
btot_sim = np.sqrt(bx_sim**2 + by_sim**2 + bz_sim**2)
   
ex_sim = simData(dat[0:,5])*econvert
ey_sim = simData(dat[0:,6])*econvert
ez_sim = simData(dat[0:,7])*econvert

uex_sim = simData(dat[0:,8])*vconvert
uey_sim = simData(dat[0:,9])*vconvert
uez_sim = simData(dat[0:,10])*vconvert

uix_sim = simData(dat[0:,11])*vconvert
uiy_sim = simData(dat[0:,12])*vconvert
uiz_sim = simData(dat[0:,13])*vconvert

ne_sim = simData(dat[0:,14])*nconvert
ni_sim = simData(dat[0:,15])*nconvert

te_perp_sim = simData(dat[0:,16])
te_para_sim = simData(dat[0:,17])

te_sim = np.sqrt(te_perp_sim**2 + te_para_sim**2)

ti_perp_sim = simData(dat[0:,18])
ti_para_sim = simData(dat[0:,19])

dbdx_sim = simData(dat[0:,20])
dbdz_sim = simData(dat[0:,21])

dbxdx_sim = simData(dat[0:,22])
dbydx_sim = simData(dat[0:,23])
dbzdx_sim = simData(dat[0:,24])

dbxdz_sim = simData(dat[0:,25])

dbydz_sim = simData(dat[0:,26])
dbzdz_sim = simData(dat[0:,27])

gradBx = np.gradient(bx_sim)
gradBy = np.gradient(by_sim)
gradBz = np.gradient(bz_sim)

dbxdx = gradBx[1]
dbxdz = gradBx[0]

dbydx = gradBy[1]
dbydz = gradBy[0]

dbzdx = gradBz[1]
dbzdz = gradBz[0]

j_sim = ni_sim*(uix_sim+uiy_sim+uiz_sim)-ne_sim*(uex_sim+uey_sim+uez_sim)
jx_sim = (ni_sim*(uix_sim)-ne_sim*(uex_sim))*jconvert/1e3
jy_sim = (ni_sim*(uiy_sim)-ne_sim*(uey_sim))*jconvert/1e3
jz_sim = (ni_sim*(uiz_sim)-ne_sim*(uez_sim))*jconvert/1e3


ex = gaussian_filter(ex_sim,.5,order=0)
ey = gaussian_filter(ey_sim,.5,order=0)
ez = gaussian_filter(ez_sim,.5,order=0)

uecrossby = (uez_sim*bx_sim-uex_sim*bz_sim)/1e3#*ucrossb_convert
uecrossbx = (uey_sim*bz_sim-uez_sim*by_sim)/1e3#*ucrossb_convert
uecrossbz = (uex_sim*by_sim-uey_sim*bz_sim)/1e3#*ucrossb_convert

eprimex = ex_sim+uecrossbx
eprimey = ey_sim+uecrossby
eprimez = ez_sim+uecrossbz

jdep = jx_sim*eprimex+jy_sim*eprimey+jz_sim*eprimez
jdepy = (jy_sim*eprimey)


#%%
epy = gaussian_filter(eprimey,10,0) 

z_center = np.where(zz==0)[0][0]
x_center = np.where(xx==0)[0][0]

x = np.linspace(0,len(xx),len(xx))
z = np.linspace(0,len(zz),len(zz))

zoff = 30
xoff = 100

xs = xx[x_center-xoff:x_center+xoff]
zs = zz[z_center-zoff:z_center+zoff]

window = epy[z_center-zoff:z_center+zoff,x_center-xoff:x_center+xoff]

z_cut = np.mean(window,axis=1)
x_cut = savgol_filter(np.mean(window,axis=0),11,0)

z_left = zs[np.where(z_cut > 0)[0][0]]
z_right = zs[np.where(z_cut > 0)[0][-1]]

x_left = xs[np.where(x_cut > 0)[0][0]]
x_right = xs[np.where(x_cut > 0)[0][-1]]
x_inds = np.where(x_cut > 0)[0]



z_height = abs(z_left) + abs(z_right)
print('z   ',z_height)
x_width = abs(x_left) + abs(x_right)
print('x   ',x_width)

aspect_ratio_2d = z_height/x_width
print(aspect_ratio_2d)



#%%
from matplotlib import pyplot as plt
import numpy as np


cmap = 'coolwarm'
fig, ax = plt.subplots(1,1)
fig.set_size_inches(5,2)

#fig.set_size_inches(8,4)

epy_norm=mpl.colors.Normalize(vmin=-4,vmax=4)

ax.contour(X,Y,Ay['ay_t50'],levels=95,colors='black',linestyles='solid',linewidths=0.2)
ax.set_title(r'$E_M+(\vec{V_e} \times \vec{B})_M$',fontsize=12)#,labelpad=8)

#ax.plot(vpx,vpz,color='black',lw=1.5,ls='-',label='11 July')
ax.plot(july11_vprobe['x']/rconvert,july11_vprobe['z']/rconvert,color='black',lw=1.5,ls='-',label='11 July')


ax.plot(aug10_vprobe['x'].sort_values()-2,aug10_vprobe['z'].sort_values(),color='black',lw=1.5,ls=':',label='10 Aug')
ax.plot(june17_vprobe['x']+10,june17_vprobe['z'],color='black',lw=1.5,ls='--', dashes=(2, 2),label='17 June')#,ls=(0, (3, 10, 1, 10)))

cax = ax.pcolormesh(xx,zz,gaussian_filter(eprimey,2,0) ,cmap=cmap,norm=epy_norm)

ax.fill_between(xs[x_inds],z_left,z_right,alpha=0.15,color='black')


ax.annotate(text=r'\bf{Central EDR}', xy=(-1.5, 4.5), xycoords='data', fontsize=10, color='black',ha='center',weight='bold')  # ,color='blue')

        
ax.legend(loc='upper left',fontsize=7)#,ncol=3)

cbar = fig.colorbar(cax)
cbar.set_label('[mV/m]')

ax.set_yticks([-10,-5,0,5,10])
ax.set_yticklabels([-10,5,0,5,10],fontsize=12)

ax.set_xticks([-60,-40,-20,0,20,40,60])
ax.set_xticklabels([-60,-40,-20,0,20,40,60],fontsize=12)

ax.set_xlim(-62,62)
ax.set_ylim(-12,12)
ax.tick_params(axis='both',which='major',length=4.5,width=1)

#ax.minorticks_on()

ax.set_ylabel('N/d$_e$',fontsize=14)
ax.set_xlabel('L/d$_e$',fontsize=14)

#fig.savefig('figure2.png',dpi=400,bbox_inches='tight')


#%%

err_dbzdx = 0.1/12
sm_pix = 2

def get_evals_err(err):
    a_evals = [];
    mddb_arr = []
    for i in range(0,len(zz)):
        mddb_row = []
        a_evals_row = []
        for j in range(0,len(xx)):
            gradbii = np.asarray([[dbxdx_sim[i][j],0,dbxdz_sim[i][j]],
                                  [dbydx_sim[i][j],0,dbydz_sim[i][j]],
                                  [dbzdx_sim[i][j]+err,0,dbzdz_sim[i][j]]])
    
            gradgradbii = np.matmul(gradbii,gradbii.T)
                
            evals_tmp = np.linalg.eig(gradgradbii)[0]
            evecs_tmp = np.linalg.eig(gradgradbii)[1].T
                
            sort_ind = np.argsort(evals_tmp)[::-1]
                
            a_evals_row.append(np.power(evals_tmp[sort_ind[1]]/evals_tmp[sort_ind[0]],0.25))

            sort_ind = np.argsort(evals_tmp)[::-1]
                    
            lc = int(eval_levicivita(sort_ind[0],sort_ind[1],sort_ind[2]))
            d0 = np.linalg.det(evecs_tmp)
            rlh = d0*lc
            
            evec_mat = np.asarray([evecs_tmp[sort_ind[0]],evecs_tmp[sort_ind[1]],evecs_tmp[sort_ind[2]]])  
            evals_mat = np.asarray([evals_tmp[sort_ind[0]],evals_tmp[sort_ind[1]],evals_tmp[sort_ind[2]]])      
        
            gradbii_mddb = np.matmul(np.matmul(evecs_tmp,gradbii),evecs_tmp.T)#.T
            mddb_row.append(gradbii_mddb)

        a_evals.append(np.asarray(a_evals_row))
        mddb_arr.append(np.asarray(mddb_row))

    return np.asarray(a_evals),np.asarray(mddb_arr)

aspectRatio_evals = get_evals_err(0)[0]

aspectRatio_evals_err = get_evals_err(err_dbzdx)[0]

#%%

fixedAR = gaussian_filter(np.sqrt(abs(dbzdx_sim)/abs(dbxdz_sim)),sm_pix,order=0)

fixedAR_err = gaussian_filter(np.sqrt((abs(dbzdx_sim+err_dbzdx))/(abs(dbxdz_sim))),sm_pix)

evalAR = gaussian_filter(aspectRatio_evals,sm_pix,order=0)
evalAR_err = gaussian_filter(aspectRatio_evals_err,sm_pix,order=0)

theory = aspect_ratio_2d
fixedAR_diff = gaussian_filter(np.abs((theory - fixedAR)/theory),sm_pix)
evalAR_diff = gaussian_filter(np.abs((theory - evalAR)/theory),sm_pix);


fixedAR_diff_err = gaussian_filter(np.abs((theory - fixedAR_err)/theory),sm_pix)
evalAR_diff_err = gaussian_filter(np.abs((theory - evalAR_err)/theory),sm_pix)

#%%
if convert == False:
    bx_nT = gaussian_filter(bx_sim*12,20,0)
    by_nT = gaussian_filter(by_sim*12,20,0)
    bz_nT = gaussian_filter(bz_sim*12,20,0)
else:
    bx_nT = gaussian_filter(bx_sim,20,0)
    by_nT = gaussian_filter(by_sim,20,0)
    bz_nT = gaussian_filter(bz_sim,20,0)   

def vprobe(arr,vprobe_x,vprobe_z):
    
    f = interp1d(np.linspace(0,len(vprobe_x),len(vprobe_x)),vprobe_z,bounds_error=False)
   
    xnew = np.linspace(0,len(vprobe_x),len(xx))
    ynew = f(xnew)   
    
    idx = []
    arr_vprobe = []
    for i in range(0,len(xnew)): #xnew
        dist = np.abs(zz[:,None]-ynew[i]) #ynew
        idx = np.argmin(dist)
        #print(idx)
        arr_vprobe.append(arr[idx][i])
    return np.asarray(arr_vprobe), xnew

bx_vp = savgol_filter(vprobe(bx_nT,vpx,vpz)[0],21,0)
by_vp = savgol_filter(vprobe(by_nT,vpx,vpz)[0],21,0)
bz_vp = savgol_filter(vprobe(bz_nT,vpx,vpz)[0],21,0)

jx_vp = savgol_filter(vprobe(jx_sim,vpx,vpz)[0],21,0)
jy_vp = savgol_filter(vprobe(jy_sim,vpx,vpz)[0],21,0)
jz_vp = savgol_filter(vprobe(jz_sim,vpx,vpz)[0],21,0)

bx_vp = savgol_filter(vprobe(bx_nT,vpx,vpz)[0],71,0)
by_vp = savgol_filter(vprobe(by_nT,vpx,vpz)[0],71,0)
bz_vp = savgol_filter(vprobe(bz_nT,vpx,vpz)[0],71,0)#,xn,mean_arr)

sim_xline = np.where(bz_vp>=0)[0][0]
sim_max_jy = np.argmax(jy_vp)


#%%
fixedAR_vp = savgol_filter(vprobe(fixedAR,vpx,vpz)[0],21,1)
evalAR_vp = savgol_filter(vprobe(evalAR,vpx,vpz)[0],21,1)

fixedAR_err_vp = savgol_filter(vprobe(fixedAR_err,vpx,vpz)[0],21,1)
evalAR_err_vp = savgol_filter(vprobe(evalAR_err,vpx,vpz)[0],21,1)


#%%
import matplotlib.colors

from mycolorpy import colorlist as mcp

cmap = matplotlib.colors.LinearSegmentedColormap.from_list('', ["#1984c5", "#22a7f0", "#63bff0", "#a7d5ed", "#e2e2e2", "#e1a692", "#de6e56", "#e14b31", "#c23728"])
norm= mpl.colors.Normalize(vmin=0.0,vmax=0.5)
levels = np.linspace(0,0.5,6)

cmap = 'coolwarm'

mm = 1/25.4  # centimeters in inches
#Full page = 190 mm x 230 mm

fig = plt.figure(constrained_layout=True,figsize= (178*mm,130*mm))


outer_grid = gridspec.GridSpec(2,1,height_ratios=[1,0.3])


outer_grid.update(hspace=1)


top_fig = outer_grid[0,0] # the left SubplotSpec within outer_grid



top_grid = gridspec.GridSpecFromSubplotSpec(2,4, top_fig,height_ratios=[1,1],width_ratios=[0.1,1,1,0.05])


#===========================================
#         
axa = fig.add_subplot(top_grid[0,1])
axa.set_title(r'No Added $\nabla{\vec{B}}$ Error',pad=10,fontsize=11)


cax = axa.contourf(xx,zz,fixedAR_diff,cmap=cmap,levels=levels,extend='both',norm=norm)
axa.contour(X,Y,Ay['ay_t50'],levels=55,colors='black',linestyles='solid',linewidths=0.25)
axa.plot(july11_vprobe['x']/rconvert,july11_vprobe['z']/rconvert,color='black',lw=1.5,ls='-')
axa.fill_between(xs[x_inds],z_left,z_right,alpha=0.2,color='black')

#axa.annotate(text=r'\bf{MMS Trajectory}', xy=(0.335, 0.415), xycoords='axes fraction',  fontsize=8,weight='bold', color='black',rotation=5.8)  # ,color='blue')
axa.annotate(text=r'\bf{MMS}', xy=(28,1), xycoords='data',  fontsize=9,weight='bold', color='black')  # ,color='blue')
axa.annotate(text=r'\bf{Central EDR}', xy=(-22,4), xycoords='data',  fontsize=9,weight='bold', color='black')  # ,color='blue')


axa.axes.get_xaxis().set_visible(False)


axa.set_ylabel('N/d$_e$',fontsize=10)

axa.set_xlim(-55,55)
axa.set_ylim(-8,8)

axa.set_yticks(np.linspace(-5,5,3)) 
axa.set_yticklabels([-5,0,5],fontsize=10)
axa.set_xticks([-40,-20,0,20,40])
axa.set_xticklabels([])


#===========================================
          
#@@@axc = fig.add_subplot(top_grid[1,0])

axc = fig.add_subplot(top_grid[1,1])


axc.contourf(xx,zz,evalAR_diff,cmap=cmap,levels=levels,extend='both',norm=norm)
axc.contour(X,Y,Ay['ay_t50'],levels=55,colors='black',linestyles='solid',linewidths=0.25)
axc.plot(july11_vprobe['x']/rconvert,july11_vprobe['z']/rconvert,color='black',lw=1.5,ls='-')
#axc.fill_between(xx[x_center-80:x_center+70],-3.12,3.10,alpha=0.25,color='black')
axc.fill_between(xs[x_inds],z_left,z_right,alpha=0.2,color='black')

axc.set_xlabel('L/d$_e$',fontsize=10)
axc.set_ylabel('N/d$_e$',fontsize=10)

axc.set_xlim(-55,55)
axc.set_ylim(-8,8)
axc.minorticks_off()

axc.set_yticks(np.linspace(-5,5,3)) 
axc.set_yticklabels([-5,0,5],fontsize=10)

axc.set_xticklabels([-40,-20,0,20,40],fontsize=10)
axc.set_xticks([-40,-20,0,20,40])

#===========================================
#          
axb = fig.add_subplot(top_grid[0,2])
axb.set_title(r'  Upper-Bound'+'\n'+r'MMS-like Added $\nabla{\vec{B}}$ Errors',pad=5,fontsize=11)


axb.contourf(xx,zz,fixedAR_diff_err,cmap=cmap,levels=levels,extend='both',norm=norm)
axb.contour(X,Y,Ay['ay_t50'],levels=55,colors='black',linestyles='solid',linewidths=0.25)
axb.plot(july11_vprobe['x']/rconvert,july11_vprobe['z']/rconvert,color='black',lw=1.5,ls='-')
axb.fill_between(xs[x_inds],z_left,z_right,alpha=0.2,color='black')

axb.set_xlim(-55,55)
axb.set_ylim(-8,8)

axb.set_yticks(np.linspace(-5,5,3)) 
axb.set_yticklabels([])
axb.set_xticklabels([])
axb.set_xticks([-40,-20,0,20,40])
axb.axes.get_xaxis().set_visible(False)

#_________________________________________
#           Fixed LMN label

ax_fixedLMN_label = fig.add_subplot(top_grid[0,0])
ax_fixedLMN_label.text(-2, 0.1, 'Fixed LMN\n   Method', rotation=90,fontsize=12,weight='roman',color='blue')
ax_fixedLMN_label.axis('off')

#_________________________________________
#           Eigenvalue Ratio label
ax_eval_label = fig.add_subplot(top_grid[1,0])
ax_eval_label.text(-2, -0.1, '  Eigenvalue\n  Ratio Method', rotation=90,fontsize=12,weight='roman',color='red')
ax_eval_label.axis('off')

#===========================================
#          
axd = fig.add_subplot(top_grid[1,2])

axd.contourf(xx,zz,evalAR_diff_err,cmap=cmap,levels=levels,extend='both',norm=norm)
axd.contour(X,Y,Ay['ay_t50'],levels=55,colors='black',linestyles='solid',linewidths=0.25)
axd.plot(july11_vprobe['x']/rconvert,july11_vprobe['z']/rconvert,color='black',lw=1.5,ls='-')
axd.fill_between(xs[x_inds],z_left,z_right,alpha=0.2,color='black')

axd.set_xlim(-55,55)
axd.set_ylim(-8,8)
axd.set_xlabel('L/d$_e$',fontsize=10)

axd.set_yticks(np.linspace(-5,5,3)) 
axd.set_yticklabels([])
axd.set_xticklabels([-40,-20,0,20,40],fontsize=10)
axd.set_xticks([-40,-20,0,20,40])

plot_labels = ['a','b','c','d']


for ii, ax in enumerate([axa,axb,axc,axd]):
    ax.text(0.05, 0.85,plot_labels[ii],horizontalalignment='center', verticalalignment='center',
            transform = ax.transAxes,size=10,bbox=dict(boxstyle="square",fc='white'))


bot_fig = outer_grid[1,0] # the left SubplotSpec within outer_grid

bot_grid = gridspec.GridSpecFromSubplotSpec(1,4, bot_fig,width_ratios=[0.1,1,1,0.05])





ar1 = fig.add_subplot(bot_grid[0,1],sharex=axc)
ar1.set_title(r'No Added $\nabla{\vec{B}}$ Error',pad=10,fontsize=11)
#ar1.text(0.5, 0.5, 'a', ha='center', va='center', size=12)#±

ar1.plot(xx,fixedAR_vp,color='blue')
ar1.plot(xx,evalAR_vp,color='red')

ar1.fill_between(xx,theory-theory*0.2,theory+theory*0.2,color='#88cef1',alpha=0.6,lw=0)
ar1.axhline(theory,ls='--',color='black',lw=0.8)
ar1.set_xticklabels([-40,-20,0,20,40],fontsize=10)
ar1.set_xticks([-40,-20,0,20,40])

ar1.set_yticklabels([0,0.2,0.5],fontsize=10)
ar1.set_yticks([0,0.2,0.5])

ar1.set_ylim(0,0.6)
ar1.set_xlabel('L/d$_e$',fontsize=10)
ar1.tick_params(axis='both',which='both',direction='in',right=True,bottom=True,top=True)
ar1.tick_params(axis='both',which='major',length=4.5,width=1)
ar1.set_ylabel('Aspect Ratio',fontsize=10)

ar1.minorticks_on()
ar1.set_xlabel('L/d$_e$',fontsize=10)

ar1.axvline(xx[sim_max_jy],ls='-',color='black',lw=1)
ar1.axvline(xx[sim_xline],ls='-',color='gray',lw=1)

trans10 = ar1.get_xaxis_transform()
ar1.text(xx[sim_max_jy+5], 0.76, 'Max J$_{M}$',color='black', fontsize=10,transform=trans10)
ar1.text(xx[sim_xline-115], 0.76, 'X-line',color='gray', fontsize=10,transform=trans10)








ar2 = fig.add_subplot(bot_grid[0,2],sharex=axd)

ar2.set_title(r'  Upper-Bound'+'\n'+r'MMS-like Added $\nabla{\vec{B}}$ Errors',pad=5,fontsize=11)

#ar2.text(0.5, 0.5, 'a', ha='center', va='center', size=12)#±
ar2.minorticks_on()

ar2.plot(xx,fixedAR_err_vp,color='blue')
ar2.plot(xx,evalAR_err_vp,color='red')

ar2.fill_between(xx,theory-theory*0.2,theory+theory*0.2,color='#88cef1',alpha=0.6,lw=0) #'#9ebeff'
ar2.axhline(theory,ls='--',color='black',lw=0.8)
ar2.set_ylim(0,0.6)

ar2.set_yticklabels([])
ar2.set_yticks([0,0.2,0.5])

ar2.set_xticklabels([-40,-20,0,20,40])
ar2.set_xticks([-40,-20,0,20,40])

ar2.tick_params(axis='both',which='both',direction='in',right=True,bottom=True,top=True)
ar2.tick_params(axis='both',which='major',length=4.5,width=1)
ar2.set_xlabel('L/d$_e$',fontsize=10)

ar2.axvline(xx[sim_max_jy],ls='-',color='black',lw=1)
ar2.axvline(xx[sim_xline],ls='-',color='gray',lw=1)

trans10 = ar2.get_xaxis_transform()
ar2.text(xx[sim_max_jy+5], 0.76, 'Max J$_{M}$',color='black', fontsize=10,transform=trans10)
ar2.text(xx[sim_xline-115], 0.76, 'X-line',color='gray', fontsize=10,transform=trans10)

ar2.annotate(text='Fixed LMN', xy=(1.05, 0.8), xycoords='axes fraction',  fontsize=10, color='blue')  # ,color='blue')
ar2.annotate(text='Eigenvalue\n     Ratio', xy=(1.05, 0.2), xycoords='axes fraction', fontsize=10, color='red') 


plot_labels = ['a','b','c','d']


for ii, ax in enumerate([axa,axb,axc,axd]):
    ax.text(0.05, 0.85,plot_labels[ii],horizontalalignment='center', verticalalignment='center',
            transform = ax.transAxes,size=10,bbox=dict(boxstyle="square",fc='white'))

plot_labels2 = ['e','f']

for ii, ax in enumerate([ar1,ar2]):
    ax.text(0.05, 0.75,plot_labels2[ii],horizontalalignment='center', verticalalignment='center',
            transform = ax.transAxes,size=10,bbox=dict(boxstyle="square",fc='white'))

#^^^^^^^^^^^^^^^^^^^^^^#^^^^^^^^^^^^^^^^^^^^^^#^^^^^^^^^^^^^^^^^^^^^^#^^^^^^^^^^^^^^^^^^^^^^#


#===========================================
#           COLORBAR
cbax = fig.add_subplot(top_grid[0:3,3])
cb = fig.colorbar(cax,orientation='vertical',cax=cbax,extendrect=True)
cb.set_label(r'Error in Aspect Ratio',fontsize=10)
cbax.set_yticklabels([r'$ 0\%$', '$ 10\%$', '$ 20\%$' ,'$ 30\%$', '$ 40\%$' ,'$\geq50\%$'],fontsize=9)

#---- --- --- --- --- --- TITLES


top_title = gridspec.GridSpec(1, 1)


top_title.update(bottom=0.995,left=0.05, top=1,  right=1-0.05)
top_title_ax = fig.add_subplot(top_title[0])
top_title_ax.text(0.5, 0.8, r'\bf{Aspect Ratio Uncertainity}', ha='center', va='center',  weight='semibold',size=13)#±
top_title_ax.axis('off')


bot_title = gridspec.GridSpec(1, 1)

bot_title.update(bottom=0.32,left=0.05, top=0.35,  right=1-0.05)
bot_title_ax = fig.add_subplot(bot_title[0])
bot_title_ax.text(0.5, 0.8, r'\bf{Cut of Aspect Ratio Along MMS Trajectory}', ha='center', va='center', weight='semibold',size=13)#±
bot_title_ax.axis('off')


#------- --- --- --- --- BOXES

top_box_grid = gridspec.GridSpec(1, 1)

top_box_grid.update(bottom=0.38,left=0.02,top=1-0.04,  right=1.03)

top_box_ax = fig.add_subplot(top_box_grid[0])
top_box_ax.tick_params(axis='both',which='both',bottom=0,left=0,labelbottom=0, labelleft=0)
top_box_ax.set_facecolor('white')
top_box_ax.patch.set_alpha(0)


bot_box_grid = gridspec.GridSpec(1, 1)

bot_box_grid.update(bottom=0.01,left=0.02,top=0.31,  right=1.03)

bot_box_ax = fig.add_subplot(bot_box_grid[0])
bot_box_ax.tick_params(axis='both',which='both',bottom=0,left=0,labelbottom=0, labelleft=0)
bot_box_ax.set_facecolor('white')
bot_box_ax.patch.set_alpha(0)

for ax in [top_box_ax,bot_box_ax]:
    for axis in ['top','bottom','left','right']:
        ax.spines[axis].set_linewidth(1.2)


#fig.savefig('figure1.png',dpi=400,bbox_inches='tight')
